
-- Copyright (c) 2001 Tait Electronics Ltd, All rights reserved. 
-- 
-- File Name:	$RCSfile: Fm_Cpu_V.vhd,v $
--
-- Revision: 	$Revision: 1.6 $
--
-- Date:		$Date: 2004/03/02 23:53:34 $
--
-- Author: 		Brian McCabe
--
-- Last Worked:	$ $
--
--
-- Description:
--
--
-- Logic Resource Use: 
-- 
--
--
-- Cases of Use:
--
-- 
-- Algorithm:
--
-- 
-- Inputs: 
--
-- 
-- Outputs:
--
-- 
-- Parameters:
--
--
-- Dependencies:
--
-- 
-- Exceptions:
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use work.amba.all;

entity Fm_Cpu_V is
    port (
        -- System Signals
        cpu_resetn   : in    std_logic; 			            
        cpu_clk      : in    std_logic;
    
        cpu_errorn   : out   std_logic;
        cpu_address  : out   std_logic_vector(21 downto 0); 	
    
        cpu_data     : inout std_logic_vector(15 downto 0);    -- 16bit external space config.
    
        cpu_brdyn    : in    std_logic;
        cpu_bexcn    : in    std_logic;
        
        -- Memory Control
    	cpu_ramcs0   : out	 std_logic;                         -- ram chip select - bank 1
        cpu_ramwe0 	 : out	 std_logic;                         
    	cpu_ramoe0	 : out	 std_logic;                         
    	cpu_rambe0	 : out	 std_logic;                         -- ram byte lane enable 0 - d[31..24]
    	cpu_rambe1   : out	 std_logic;                         -- ram byte lane enable 1 - d[23..16] 
        cpu_rambe2   : out	 std_logic;                         -- ram byte lane enable 2 - d[15..8]
        cpu_rambe3   : out	 std_logic;                         -- ram byte lane enable 3 - d[7..0]  
    
        cpu_romcs    : out   std_logic;
        cpu_romwe    : out   std_logic;
        cpu_romoe    : out   std_logic;
        
        -- Parallel IO
        cpu_pio      : inout std_logic_vector(15 downto 0); 	
    
        -- Watchdog
        cpu_wdogn    : out   std_logic;				            
    
        -- Serial Debug Interface
        cpu_dsuen    : in    std_logic;
        cpu_dsutx    : out   std_logic;
        cpu_dsurx    : in    std_logic;
        cpu_dsubre   : in    std_logic;
        cpu_dsuact   : out   std_logic;
        cpu_test     : in    std_logic;
    
        cpu_ext_irq  : in    std_logic_vector(1 downto 0);
        
        cpu_ext_apbi : out   apb_slv_in_vector(0 to 3); 
        cpu_ext_apbo : in    apb_slv_out_vector(0 to 3)
    );
end;

architecture rtl of Fm_Cpu_V is

component leon
    port (
        resetn   : in    std_logic; 			            
        clk      : in    std_logic;
    
        errorn   : out   std_logic;
        address  : out   std_logic_vector(22 downto 0); 	
    
        data     : inout std_logic_vector(31 downto 16);
    
        brdyn    : in    std_logic;
        bexcn    : in    std_logic;
        
    	ramcs0   : out	 std_logic;
        ramwe0 	 : out	 std_logic;                         
    	ramoe0	 : out	 std_logic;
    	rambe0	 : out	 std_logic;
    	rambe1   : out	 std_logic;
        rambe2   : out	 std_logic;
        rambe3   : out	 std_logic;
        
        romcs    : out   std_logic;
        romwe    : out   std_logic;
        romoe    : out   std_logic;
        
        pio      : inout std_logic_vector(15 downto 0); 	
    
        wdogn    : out   std_logic;				            
    
        dsuen    : in    std_logic;
        dsutx    : out   std_logic;
        dsurx    : in    std_logic;
        dsubre   : in    std_logic;
        dsuact   : out   std_logic;
        test     : in    std_logic;
    
        ext_irq  : in    std_logic_vector(1 downto 0);
        
        ext_apbi : out   apb_slv_in_vector(0 to 3); 
        ext_apbo : in    apb_slv_out_vector(0 to 3)
    );
end component;

begin
    cpu0 : leon
        port map (
            resetn => cpu_resetn,  
            clk => cpu_clk,    
                                       
            errorn => cpu_errorn,  
            address(22 downto 1) => cpu_address, 
                                       
            data => cpu_data,    
                                       
            brdyn => cpu_brdyn,   
            bexcn => cpu_bexcn,   
                                       
            ramcs0 => cpu_ramcs0,
            ramwe0 => cpu_ramwe0,     
            ramoe0 => cpu_ramoe0,     
            rambe0 => cpu_rambe0,     
            rambe1 => cpu_rambe1,     
            rambe2 => cpu_rambe2,     
            rambe3 => cpu_rambe3,     
                  
            romcs => cpu_romcs,
            romwe => cpu_romwe, 
            romoe => cpu_romoe, 
                                
            pio => cpu_pio,             
                            
            wdogn => cpu_wdogn,    
                                       
            dsuen => cpu_dsuen, 
            dsutx => cpu_dsutx, 
            dsurx => cpu_dsurx, 
            dsubre => cpu_dsubre,
            dsuact => cpu_dsuact,
            
            test => cpu_test,
            
            ext_irq => cpu_ext_irq,
            
            ext_apbi => cpu_ext_apbi,
            ext_apbo => cpu_ext_apbo                 
        );                  
         
end;                         
