



----------------------------------------------------------------------------
--  This file is a part of the LEON VHDL model
--  Copyright (C) 1999  European Space Agency (ESA)
--
--  This library is free software; you can redistribute it and/or
--  modify it under the terms of the GNU Lesser General Public
--  License as published by the Free Software Foundation; either
--  version 2 of the License, or (at your option) any later version.
--
--  See the file COPYING.LGPL for the full details of the license.


-----------------------------------------------------------------------------
-- Entity: 	ahbram
-- File:	ahbram.vhd
-- Author:	Jiri Gaisler - Gaisler Reserch
-- Description:	AHB ram. 0-waitstate read, 0/1-waitstate write.
------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.conv_integer;
use work.target.all;
use work.config.all;
use work.iface.all;
use work.amba.all;
use work.tech_map.all;

entity ahbram is
  generic ( abits : integer := 10);
  port (
    rst    : in  std_logic;
    clk    : in  clk_type;
    ahbsi  : in  ahb_slv_in_type;
    ahbso  : out ahb_slv_out_type
  );
end;

architecture rtl of ahbram is
type reg_type is record
  write  : std_logic;
  hready : std_logic;
  addr   : std_logic_vector(abits-1 downto 0);
  size   : std_logic_vector(1 downto 0);
end record;
signal r, c : reg_type;
signal ramsel : std_logic;
signal write : std_logic_vector(3 downto 0);
signal ramaddr  : std_logic_vector(abits-1 downto 0);
begin

  comb : process (ahbsi, r, rst)
  variable bs : std_logic_vector(3 downto 0);
  variable hready, hsel : std_logic;
  variable v : reg_type;
  variable haddr  : std_logic_vector(abits-1 downto 0);
  begin
    v := r; bs := (others => '0'); hready := '1'; v.write := '0';
    haddr := ahbsi.haddr(abits+1 downto 2); hsel := '0';
    if (ahbsi.hready and ahbsi.hsel and ahbsi.htrans(1)) = '1' then
      v.write := ahbsi.hwrite; hsel := not ahbsi.hwrite;
      v.addr := ahbsi.haddr(abits+1 downto 2); 
      v.size := ahbsi.hsize(1 downto 0);
    end if;
    if r.write = '1' then
      case r.size(1 downto 0) is
      when "00" => bs (conv_integer(r.addr(1 downto 0))) := '1';
      when "01" => bs := r.addr(1) & r.addr(1) & not (r.addr(1) & r.addr(1));
      when others => bs := (others => '1');
      end case;
      haddr := r.addr; hsel := '1';
      hready := ahbsi.hwrite or not ahbsi.htrans(1) or not ahbsi.hsel; 
    end if;
    if rst = '0' then v.write := '0'; end if;
    write <= bs; ramsel <= hsel; ahbso.hready <= hready; 
    ramaddr <= haddr; c <= v;
  end process;

  ahbso.hresp <= "00"; ahbso.hsplit <= (others => '0');

  --ra : for i in 0 to 3 generate
    --aram :  ahbsyncram generic map (abits, 8) port map (
    	--ramaddr, clk, ahbsi.hwdata(i*8+7 downto i*8),
    	--ahbso.hrdata(i*8+7 downto i*8), ramsel, write(3-i)); 
  --end generate;
    aram : ahbsyncram
        generic map (
                    abits,
                    32
        ) 
        port map    (
    	            ramaddr,
                    clk,
                    ahbsi.hwdata,
    	            ahbso.hrdata,
                    ramsel,
                    write
        ); 

  reg : process (clk)
  begin
    if rising_edge(clk ) then r <= c; end if;
  end process;
end;
