



----------------------------------------------------------------------------
--  This file is a part of the LEON VHDL model
--  Copyright (C) 1999  European Space Agency (ESA)
--
--  This library is free software; you can redistribute it and/or
--  modify it under the terms of the GNU Lesser General Public
--  License as published by the Free Software Foundation; either
--  version 2 of the License, or (at your option) any later version.
--
--  See the file COPYING.LGPL for the full details of the license.


-----------------------------------------------------------------------------
-- Entity: 	cachemem
-- File:	cachemem.vhd
-- Author:	Jiri Gaisler - ESA/ESTEC
-- Description:	Contains ram cells for both instruction and data caches
------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use work.target.all;
use work.config.all;
use work.iface.all;
use work.macro.all;
use work.tech_map.all;

entity cachemem is
  port (
        clk   : in  clk_type;
	    crami : in  cram_in_type;
	    cramo : out cram_out_type
  );
end;

architecture rtl of cachemem is

  constant ITDEPTH : natural := 2**IOFFSET_BITS;
  --BSM:Step1: constant DTDEPTH : natural := 2**DOFFSET_BITS;

  constant ITWIDTH : natural := ITAG_BITS + ILRR_BIT + ICLOCK_BIT;
  --BSM:Step1: constant DTWIDTH : natural := DTAG_BITS + DLRR_BIT + DCLOCK_BIT;

  --BSM:Step1: subtype dtdatain_vector is std_logic_vector(DTWIDTH downto 0);
  --BSM:Step1: type dtdatain_type is array (0 to MAXSETS-1) of dtdatain_vector;
  
  subtype itdataout_vector is std_logic_vector(ITWIDTH downto 0);
  type itdataout_type is array (0 to MAXSETS-1) of itdataout_vector;
  subtype iddataout_vector is std_logic_vector(32 -1 downto 0);
  type iddataout_type is array (0 to MAXSETS-1) of iddataout_vector;
  --BSM:Step1: subtype dtdataout_vector is std_logic_vector(DTWIDTH downto 0);
  --BSM:Step1: type dtdataout_type is array (0 to MAXSETS-1) of dtdataout_vector;
  --BSM:Step1: subtype dddataout_vector is std_logic_vector(32 -1 downto 0);
  --BSM:Step1: type dddataout_type is array (0 to MAXSETS-1) of dddataout_vector;
  

  signal itaddr    : std_logic_vector(IOFFSET_BITS + ILINE_BITS -1 downto ILINE_BITS);
  signal idaddr    : std_logic_vector(IOFFSET_BITS + ILINE_BITS -1 downto 0);


  signal itdatain  : std_logic_vector(ITWIDTH -1 downto 0);
  signal itdataout : itdataout_type;
  signal iddatain  : std_logic_vector(32 -1 downto 0);
  signal iddataout : iddataout_type;

  signal itenable  : std_logic;
  signal idenable  : std_logic;
  signal itwrite   : std_logic_vector(0 to MAXSETS-1);
  signal idwrite   : std_logic_vector(0 to MAXSETS-1);

  --BSM:Step1: signal dtaddr    : std_logic_vector(DOFFSET_BITS + DLINE_BITS -1 downto DLINE_BITS);
  --BSM:Step1: signal dtaddr2   : std_logic_vector(DOFFSET_BITS + DLINE_BITS -1 downto DLINE_BITS);
  --BSM:Step1: signal ddaddr    : std_logic_vector(DOFFSET_BITS + DLINE_BITS -1 downto 0);

  --signal dtdatain  : std_logic_vector(DTWIDTH -1 downto 0);
  --BSM:Step1: signal dtdatain  : dtdatain_type; 
  --  signal dtdatain2 : std_logic_vector(DTWIDTH -1 downto 0);
  --BSM:Step1: signal dtdatain2 : dtdatain_type;
  --BSM:Step1: signal dtdataout : dtdataout_type;
  --BSM:Step1: signal dtdataout2: dtdataout_type;
  --BSM:Step1: signal dddatain  : std_logic_vector(32 -1 downto 0);
  --BSM:Step1: signal dddataout : dddataout_type;

  --BSM:Step1: signal dtenable  : std_logic;
  --BSM:Step1: signal dtenable2 : std_logic;
  --BSM:Step1: signal ddenable  : std_logic;
  --BSM:Step1: signal dtwrite   : std_logic_vector(0 to MAXSETS-1);
  --BSM:Step1: signal dtwrite2  : std_logic_vector(0 to MAXSETS-1);
  --BSM:Step1: signal ddwrite   : std_logic_vector(0 to MAXSETS-1);

  signal vcc, gnd  : std_logic;

begin

  vcc <= '1'; gnd <= '0'; 
  itaddr <= crami.icramin.idramin.address(IOFFSET_BITS + ILINE_BITS -1 downto ILINE_BITS);
  idaddr <= crami.icramin.idramin.address;


  itinsel : process(crami)
  variable vitdatain  : std_logic_vector(ITWIDTH downto 0);
  --BSM:Step1: variable vdtdatain : dtdatain_type;
  --BSM:Step1: variable vdtdatain2 : dtdatain_type;
  begin


    vitdatain := (others => '0');

    vitdatain(ITWIDTH-(ILRR_BIT + ICLOCK_BIT)) := crami.icramin.itramin.lrr;
    vitdatain(ITWIDTH-ICLOCK_BIT) := crami.icramin.itramin.lock;
    vitdatain(ITAG_BITS-1 downto 0) := crami.icramin.itramin.tag & crami.icramin.itramin.valid;
    itdatain <= vitdatain(ITWIDTH-1 downto 0);
    iddatain(31 downto 0) <= crami.icramin.idramin.data;

    --BSM:Step1: for i in 0 to MAXSETS-1 loop
      --BSM:Step1: vdtdatain(i) := (others => '0');

      --BSM:Step1: vdtdatain(i)(DTWIDTH-(DLRR_BIT + DCLOCK_BIT)) := crami.dcramin.dtramin.lrr(i);
      --BSM:Step1: vdtdatain(i)(DTWIDTH-DCLOCK_BIT) := crami.dcramin.dtramin.lock(i);          
      --BSM:Step1: vdtdatain(i)(DTAG_BITS-1 downto 0) := crami.dcramin.dtramin.tag & crami.dcramin.dtramin.valid;
    --BSM:Step1: end loop;
    --BSM:Step1: dtdatain <= vdtdatain;

    --BSM:Step1: vdtdatain2 := (others => (others => '0'));
    --BSM:Step1: for i in 0 to MAXSETS-1 loop

      --BSM:Step1: vdtdatain2(i)(DTWIDTH-(DLRR_BIT + DCLOCK_BIT)) := dtdataout2(i)(DTWIDTH-(DLRR_BIT+DCLOCK_BIT));
      --BSM:Step1: vdtdatain2(i)(DTWIDTH-DCLOCK_BIT) := dtdataout2(i)(DTWIDTH-DCLOCK_BIT);
      --BSM:Step1: vdtdatain2(i)(DTAG_BITS-1 downto DLINE_SIZE) := crami.dcramin.dtraminsn.tag;
    --BSM:Step1: end loop;
    --BSM:Step1: dtdatain2 <= vdtdatain2;
    
    --BSM:Step1: dddatain(32 - 1 downto 0) <= crami.dcramin.ddramin.data;

  end process;

  
  itwrite   <= crami.icramin.itramin.write;
  idwrite   <= crami.icramin.idramin.write;
  itenable  <= crami.icramin.itramin.enable;
  idenable  <= crami.icramin.idramin.enable;

  --BSM:Step1: dtaddr <= crami.dcramin.ddramin.address(DOFFSET_BITS + DLINE_BITS -1 downto DLINE_BITS);
  --BSM:Step1: dtaddr2 <= crami.dcramin.dtraminsn.address;
  --BSM:Step1: ddaddr <= crami.dcramin.ddramin.address;
  --BSM:Step1: dtwrite   <= crami.dcramin.dtramin.write;
  --BSM:Step1: dtwrite2  <= crami.dcramin.dtraminsn.write;
  --BSM:Step1: ddwrite   <= crami.dcramin.ddramin.write;
  --BSM:Step1: dtenable  <= crami.dcramin.dtramin.enable;
  --BSM:Step1: dtenable2 <= crami.dcramin.dtraminsn.enable;
  --BSM:Step1: ddenable  <= crami.dcramin.ddramin.enable;


  it0 : for i in 0 to ISETS-1 generate
    itags0 : syncram
      generic map ( dbits => ITWIDTH, abits => IOFFSET_BITS)
      port map ( itaddr, clk, itdatain, itdataout(i)(ITWIDTH-1 downto 0), itenable, itwrite(i));
  end generate;

  --BSM:Step1: dtags0 : if not DSNOOP generate
    --BSM:Step1: dt0 : for i in 0 to DSETS-1 generate
      --BSM:Step1: dtags0 : syncram
      --BSM:Step1: generic map ( dbits => DTWIDTH, abits => DOFFSET_BITS)
      --BSM:Step1: port map ( dtaddr, clk, dtdatain(i)(DTWIDTH-1 downto 0), dtdataout(i)(DTWIDTH-1 downto 0), dtenable, dtwrite(i));
    --BSM:Step1: end generate;
  --BSM:Step1: end generate;

  --BSM:Step1: dtags1 : if DSNOOP generate
    --BSM:Step1: dt0 : for i in 0 to DSETS-1 generate
      --BSM:Step1: dtags0 : dpsyncram
      --BSM:Step1: generic map ( dbits => DTWIDTH, abits => DOFFSET_BITS)
      --BSM:Step1: port map ( dtaddr, clk, dtdatain(i)(DTWIDTH-1 downto 0), dtdataout(i)(DTWIDTH-1 downto 0), dtenable, dtwrite(i),
                 --BSM:Step1: dtaddr2, dtdatain2(i)(DTWIDTH-1 downto 0), dtdataout2(i)(DTWIDTH-1 downto 0), dtenable2, dtwrite2(i));
    --BSM:Step1: end generate;
  --BSM:Step1: end generate;

  id0 : for i in 0 to ISETS-1 generate
    idata0 : syncram
      generic map ( dbits => 32, abits => IOFFSET_BITS+ILINE_BITS)
      port map ( idaddr, clk, iddatain, iddataout(i), idenable, idwrite(i));
  end generate;

  --BSM:Step1: dd0 : for i in 0 to DSETS-1 generate
    --BSM:Step1: ddata0 : syncram
     --BSM:Step1: generic map ( dbits => 32, abits => DOFFSET_BITS+DLINE_BITS)
      --BSM:Step1: port map ( ddaddr, clk, dddatain, dddataout(i), ddenable, ddwrite(i));
  --BSM:Step1: end generate;

  itx : for i in 0 to ISETS-1 generate
    itdataout(i)(ITWIDTH) <= '0';
    cramo.icramout.itramout(i).valid <= itdataout(i)(ILINE_SIZE -1 downto 0);
    cramo.icramout.itramout(i).tag <= itdataout(i)(ITAG_BITS-1 downto ILINE_SIZE);
    cramo.icramout.itramout(i).lrr <= itdataout(i)(ITWIDTH - (ILRR_BIT+ICLOCK_BIT));
    cramo.icramout.itramout(i).lock <= itdataout(i)(ITWIDTH-ICLOCK_BIT);     
    cramo.icramout.idramout(i).data <= iddataout(i)(31 downto 0);
  end generate;


  itd : for i in 0 to DSETS-1 generate
    --BSM:Step1: dtdataout(i)(DTWIDTH) <= '0';
    --BSM:Step1: dtdataout2(i)(DTWIDTH) <= '0';
    --BSM:Step1: cramo.dcramout.dtramout(i).valid <= dtdataout(i)(DLINE_SIZE -1 downto 0);
    cramo.dcramout.dtramout(i).valid <= (others => '0');
    --BSM:Step1: cramo.dcramout.dtramout(i).tag <= dtdataout(i)(DTAG_BITS-1 downto DLINE_SIZE);
    cramo.dcramout.dtramout(i).tag <= (others => '0');
    --BSM:Step1: cramo.dcramout.dtramout(i).lrr <= dtdataout(i)(DTWIDTH-(DLRR_BIT+DCLOCK_BIT));
    cramo.dcramout.dtramout(i).lrr <= '0';
    --BSM:Step1: cramo.dcramout.dtramout(i).lock <= dtdataout(i)(DTWIDTH-DCLOCK_BIT);
    cramo.dcramout.dtramout(i).lock <= '0';
    --BSM:Step1: cramo.dcramout.dtramoutsn(i).tag <= dtdataout2(i)(DTAG_BITS-1 downto DLINE_SIZE);
    cramo.dcramout.dtramoutsn(i).tag <= (others => '0');
    --BSM:Step1: cramo.dcramout.ddramout(i).data <= dddataout(i)(31 downto 0);
    cramo.dcramout.ddramout(i).data <= (others => '0');
  end generate;

end ;

