
-- Copyright (c) 2001 Tait Electronics Ltd, All rights reserved. 
-- 
-- File Name:	$RCSfile: tech_altera.vhd,v $
--
-- Revision: 	$Revision: 1.6 $
--
-- Date:		$Date: 2004/08/11 03:47:50 $
--
-- Author: 		Brian McCabe
--
-- Last Worked:	$ $
--
--
-- Description:
--
--
-- Logic Resource Use: 
-- 
--
--
-- Cases of Use:
--
-- 
-- Algorithm:
--
-- 
-- Inputs: 
--
-- 
-- Outputs:
--
-- 
-- Parameters:
--
--
-- Dependencies:
--
-- 
-- Exceptions:
--
--------------------------------------------------------------------------------

LIBRARY ieee;
use IEEE.std_logic_1164.all;
use work.iface.all;
use work.Sumo_TargetPak_V.all;
package tech_altera is

-- Altera sync ram

component altera_syncram
  generic ( abits : integer := 10; dbits : integer := 8 );
  port (
    address  : in std_logic_vector((abits -1) downto 0);
    clk      : in std_logic;
    datain   : in std_logic_vector((dbits -1) downto 0);
    dataout  : out std_logic_vector((dbits -1) downto 0);
    enable   : in std_logic;
    write    : in std_logic
   ); 
end component;

 --Altera AHB cync ram
component altera_ahbsyncram
  generic ( abits : integer := 10; dbits : integer := 8 );
  port (
    address  : in std_logic_vector((abits -1) downto 0);
    clk      : in std_logic;
    datain   : in std_logic_vector((dbits -1) downto 0);
    dataout  : out std_logic_vector((dbits -1) downto 0);
    enable   : in std_logic;
    write    : in std_logic_vector(3 downto 0)
   ); 
end component;

-- regfile generator

component altera_regfile_iu
  generic ( 
    rftype : integer := 1;
    abits : integer := 8; dbits : integer := 32; words : integer := 128
  );
  port (
    rst      : in std_logic;
    clk      : in std_logic;
    clkn     : in std_logic;
    rfi      : in rf_in_type;
    rfo      : out rf_out_type);
  end component;

component altera_regfile_cp
  generic ( 
    abits : integer := 4; dbits : integer := 32; words : integer := 16
  );
  port (
    rst      : in std_logic;
    clk      : in std_logic;
    rfi      : in rf_cp_in_type;
    rfo      : out rf_cp_out_type);
end component;

-- BPROM
component altera_bprom
  port (
    clk       : in std_logic;
    addr      : in std_logic_vector(8 downto 0);
    data      : out std_logic_vector(31 downto 0));
end component;

end;

library IEEE;
use IEEE.std_logic_1164.all;

-- direct instantiation of cache RAM using Altera ALTSYNCRAM library entity

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use work.iface.all;

entity altera_syncram is
  generic ( abits : integer := 10; dbits : integer := 8 );
  port (
    address  : in std_logic_vector((abits -1) downto 0);
    clk      : in std_logic;
    datain   : in std_logic_vector((dbits -1) downto 0);
    dataout  : out std_logic_vector((dbits -1) downto 0);
    enable   : in std_logic;
    write    : in std_logic
  ); 
end;     

architecture rtl of altera_syncram is
  component altsyncram
  generic (
		operation_mode		                : string;
		width_a		                        : integer;
		widthad_a		                    : integer;
		lpm_type		                    : string;
		width_byteena_a		                : integer;
		outdata_reg_a		                : string;
		outdata_aclr_a		                : string;
		indata_aclr_a		                : string;
		wrcontrol_aclr_a		            : string;
		address_aclr_a		                : string;
		read_during_write_mode_mixed_ports	: string;
		ram_block_type		                : string;
		intended_device_family		        : string
  );
  port (
   		wren_a	                            : in std_logic ;
   		clock0	                            : in std_logic ;
   		address_a	                        : in std_logic_vector (abits-1 downto 0);
   		q_a	                                : out std_logic_vector (dbits-1 downto 0);
   		data_a	                            : in std_logic_vector (dbits-1 downto 0)
  );
  end component;

begin
    u0 : altsyncram
       generic map (
    		operation_mode => "SINGLE_PORT",
    		width_a => dbits,
    		widthad_a => abits,
    		lpm_type => "altsyncram",
    		width_byteena_a => 1,
    		outdata_reg_a => "UNREGISTERED",
    		outdata_aclr_a => "NONE",
    		indata_aclr_a => "NONE",
    		wrcontrol_aclr_a => "NONE",
    		address_aclr_a => "NONE",
    		read_during_write_mode_mixed_ports => "DONT_CARE",
    		ram_block_type => "AUTO",
    		intended_device_family => "Stratix"
	   )
	   port map (
		    wren_a => write AND enable,
		    clock0 => clk,
		    address_a => address,
		    data_a => datain,
		    q_a => dataout
	   );
end;


-- direct instantiation of AHB RAM using Altera ALTSYNCRAM library entity

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use work.iface.all;
use work.Sumo_TargetPak_V.all;


entity altera_ahbsyncram is
  generic ( abits : integer := 6; dbits : integer := 32 );
  port (
    address  : in std_logic_vector((abits -1) downto 0);
    clk      : in std_logic;
    datain   : in std_logic_vector((dbits -1) downto 0);
    dataout  : out std_logic_vector((dbits -1) downto 0);
    enable   : in std_logic;
    write    : in std_logic_vector(3 downto 0)
  ); 
end;     

architecture rtl of altera_ahbsyncram is
  component altsyncram
  generic (
		operation_mode		                : string;                                     
		width_a		                        : integer;                                    
		widthad_a		                    : integer;                                    
		--numwords_a                          : integer;
        lpm_type		                    : string;                                     
		width_byteena_a		                : integer;                                    
		byte_size                           : integer;
        outdata_reg_a		                : string;                                     
		outdata_aclr_a		                : string;                                     
		indata_aclr_a		                : string;                                     
		wrcontrol_aclr_a		            : string;                                     
		address_aclr_a		                : string;                                     
		byteena_aclr_a                      : string;
        read_during_write_mode_mixed_ports	: string;                                     
		ram_block_type		                : string;                                     
		init_file                           : string;
        intended_device_family		        : string                                      
  );                                                                                      
  port (                                                                                  
   		wren_a	                            : in std_logic;
   		clock0	                            : in std_logic;
        byteena_a                           : in std_logic_vector (3 downto 0);      
   		address_a	                        : in std_logic_vector (abits-1 downto 0);     
   		q_a	                                : out std_logic_vector (dbits-1 downto 0);         
   		data_a	                            : in std_logic_vector (dbits-1 downto 0)           
  );                                                                                           
  end component;                                                                           

begin
    u0 : altsyncram
       generic map (
    		operation_mode => "SINGLE_PORT",                             
    		width_a => dbits,                                            
    		widthad_a => abits,
            --numwords_a => 2048,                                            
    		lpm_type => "altsyncram",                                    
    		width_byteena_a => 4,                                        
    		byte_size => 8,
            outdata_reg_a => "UNREGISTERED",                             
    		outdata_aclr_a => "NONE",                                    
    		indata_aclr_a => "NONE",                                     
    		wrcontrol_aclr_a => "NONE",                                  
    		address_aclr_a => "NONE",                                    
    		byteena_aclr_a => "NONE",
            read_during_write_mode_mixed_ports => "DONT_CARE",           
    		ram_block_type => "AUTO",                                    
    		init_file => FM_AHBBOOT_TYPE & ".mif", --"ahbboot.mif",
            intended_device_family => "Stratix"                          
	   )                                                                 
	   port map (                                                        
		    wren_a => enable,                                   
		    clock0 => clk,                                                
		    address_a => address,                     
		    byteena_a => write,
            data_a => datain,                         
		    q_a => dataout                            
	   );                                             
end;

-- integer unit regfile
LIBRARY ieee;
use IEEE.std_logic_1164.all;
use work.config.all;
use work.iface.all;
entity altera_regfile_iu is
  generic ( 
    rftype : integer := 1;
    abits : integer := 8; dbits : integer := 32; words : integer := 128
  );
  port (
    rst      : in std_logic;
    clk      : in std_logic;
    clkn     : in std_logic;
    rfi      : in rf_in_type;
    rfo      : out rf_out_type);
end;

architecture rtl of altera_regfile_iu is
  component altsyncram
  generic (
		operation_mode		                : string;
		width_a		                        : integer;
		widthad_a		                    : integer;
		numwords_a		                    : integer;
		width_b		                        : integer;
		widthad_b		                    : integer;
		numwords_b		                    : integer;
		lpm_type		                    : string;
		width_byteena_a		                : integer;
		outdata_reg_b		                : string;
		indata_aclr_a		                : string;
		wrcontrol_aclr_a		            : string;
		address_aclr_a		                : string;
		address_reg_b		                : string;
		address_aclr_b		                : string;
		outdata_aclr_b		                : string;
		read_during_write_mode_mixed_ports	: string;
		ram_block_type		                : string;
		intended_device_family		        : string
  );
  port (
		wren_a	                            : in std_logic ;
		clock0	                            : in std_logic ;
		address_a	                        : in std_logic_vector (abits-1 downto 0);
		address_b	                        : in std_logic_vector (abits-1 downto 0);
		q_b	                                : out std_logic_vector (dbits-1 downto 0);
		data_a	                            : in std_logic_vector (dbits-1 downto 0)
  );                                        
  end component;

signal qq1, qq2  : std_logic_vector (dbits -1 downto 0);

begin
  rfss : if rftype = 1 generate
    u0 : altsyncram 
       generic map (
    	    operation_mode => "DUAL_PORT",
    	    width_a => dbits,
    	    widthad_a => abits,
    	    numwords_a => words,
    	    width_b => dbits,
    	    widthad_b => abits,
    	    numwords_b => words,
    	    lpm_type => "altsyncram",
    	    width_byteena_a => 1,
    	    outdata_reg_b => "UNREGISTERED",
    	    indata_aclr_a => "NONE",
    	    wrcontrol_aclr_a => "NONE",
    	    address_aclr_a => "NONE",
    	    address_reg_b => "CLOCK0",
    	    address_aclr_b => "NONE",
    	    outdata_aclr_b => "NONE",
    	    read_during_write_mode_mixed_ports => "DONT_CARE",
    	    ram_block_type => "AUTO",
    	    intended_device_family => "Stratix"
       )
       port map (
            wren_a => rfi.wren,
            clock0 => clkn,
            address_b => rfi.rd1addr,
            address_a => rfi.wraddr,
            data_a => rfi.wrdata,
            q_b => rfo.data1
       );

    u1 : altsyncram 
       generic map (
    	    operation_mode => "DUAL_PORT",
    	    width_a => dbits,
    	    widthad_a => abits,
    	    numwords_a => words,
    	    width_b => dbits,
    	    widthad_b => abits,
    	    numwords_b => words,
    	    lpm_type => "altsyncram",
    	    width_byteena_a => 1,
    	    outdata_reg_b => "UNREGISTERED",
    	    indata_aclr_a => "NONE",
    	    wrcontrol_aclr_a => "NONE",
    	    address_aclr_a => "NONE",
    	    address_reg_b => "CLOCK0",
    	    address_aclr_b => "NONE",
    	    outdata_aclr_b => "NONE",
    	    read_during_write_mode_mixed_ports => "DONT_CARE",
    	    ram_block_type => "AUTO",
    	    intended_device_family => "Stratix"
       )
       port map (
            wren_a => rfi.wren,
            clock0 => clkn,
            address_b => rfi.rd2addr,
            address_a => rfi.wraddr,
	        data_a => rfi.wrdata,
            q_b => rfo.data2
       );
  end generate;

end;

-- BPROM instantiation  
library IEEE;
use IEEE.std_logic_1164.all;
use work.config.all;

entity altera_bprom is
  port (
    clk       : in std_logic;
    addr      : in std_logic_vector(8 downto 0);
    data      : out std_logic_vector(31 downto 0)
  );
end;
  
architecture rtl of altera_bprom is
  component altsyncram
  generic (
		operation_mode		                : string;
		width_a		                        : integer;
		widthad_a		                    : integer;
		lpm_type		                    : string;
		width_byteena_a		                : integer;
		outdata_reg_a		                : string;
		outdata_aclr_a		                : string;
		indata_aclr_a		                : string;
		wrcontrol_aclr_a		            : string;
		address_aclr_a		                : string;
		read_during_write_mode_mixed_ports	: string;
		ram_block_type		                : string;
		intended_device_family		        : string;
        init_file                           : string;
        numwords_a                          : integer
  );
  port (
   		--wren_a	                            : in std_logic ;
   		clock0	                            : in std_logic ;
   		address_a	                        : in std_logic_vector (8 downto 0);
   		q_a	                                : out std_logic_vector (31 downto 0)--;
   		--data_a	                            : in std_logic_vector (31 downto 0)
  );
  end component;

begin
    u0 : altsyncram
       generic map (
    		operation_mode => "ROM",
    		width_a => 32,
    		widthad_a => 9,
    		lpm_type => "altsyncram",
    		width_byteena_a => 1,
    		outdata_reg_a => "UNREGISTERED",
    		outdata_aclr_a => "NONE",
    		indata_aclr_a => "NONE",
    		wrcontrol_aclr_a => "NONE",
    		address_aclr_a => "NONE",
    		read_during_write_mode_mixed_ports => "DONT_CARE",
    		ram_block_type => "AUTO",
    		intended_device_family => "Stratix",
            init_file => "bprom.mif",
            numwords_a => 512
	   )
	   port map (
		    --wren_a => '0',
		    clock0 => clk,
		    address_a => addr,
		    --data_a => (others => '0'),
		    q_a => data
	   );
end;
